import json
import logging
from urllib.parse import urlencode
from django.conf import settings
from django.contrib import auth
from django.contrib.auth.models import User
from django.http import HttpResponse, HttpRequest
from django.shortcuts import redirect
from django.views.generic import TemplateView
from rest_framework.request import Request
from rest_framework.views import APIView
from .helpers import facebook_parse_signed_request
from .models import FacebookAccessToken, FacebookUser
from .serializers import AccountKitAuthTokenSerializer, FacebookAuthTokenSerializer, GoogleAuthTokenSerializer, \
    AuthResultSerializer


logger = logging.getLogger(__name__)


class OAuthRedirectViewBase(TemplateView):
    template_name = 'jauth/oauth-redirect.html'
    serializer_class = None
    result_serializer_class = AuthResultSerializer

    def get_context_data(self, **kw):
        request = self.request
        assert isinstance(request, HttpRequest)

        cx = {
            'auth_result_json': 'null',
            'status': 'not_authenticated',
        }
        for k, v in kw.items():
            if v:
                cx[k] = v

        data = request.GET.dict()
        if 'code' in data:
            serializer = self.serializer_class()
            data = serializer.validate(data)
            user = data['user']
            cx['user'] = user
            if settings.JAUTH_AUTHENTICATION_SUCCESS_REDIRECT:
                auth.login(request, user)
                return cx
            result = self.result_serializer_class(user).data
            cx['auth_result_json'] = json.dumps(result)
            cx['status'] = 'ok'

        return cx

    def post(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        try:
            context = self.get_context_data(**kwargs)
            if settings.JAUTH_AUTHENTICATION_SUCCESS_REDIRECT:
                return redirect(settings.JAUTH_AUTHENTICATION_SUCCESS_REDIRECT)
            return self.render_to_response(context)
        except Exception as e:
            if settings.JAUTH_AUTHENTICATION_ERROR_REDIRECT:
                return redirect(settings.JAUTH_AUTHENTICATION_ERROR_REDIRECT + '?' + urlencode({'msg': str(e)}))
            return HttpResponse(content=str(e).encode(), status=401)  # HTTP 401 Unauthorized


class FacebookDeauthorizeView(APIView):
    def post(self, request: Request, *args, **kwargs):
        logger.info('FacebookDeauthorizeView BEGIN -----------------------')
        logger.info(request.POST.dict())
        signed_request = request.POST.get('signed_request')
        data = facebook_parse_signed_request(signed_request)
        logger.info('Parsed data:')
        logger.info(data)
        user_id = data['user_id']
        FacebookAccessToken.objects.filter(ext_user__ext_user_id=user_id).delete()
        logger.info('FacebookDeauthorizeView END -------------------------')
        return data


class FacebookDeleteView(APIView):
    def post(self, request, *args, **kwargs):
        logger.info('FacebookDeleteView BEGIN ----------------------------')
        logger.info(request.POST.dict())
        signed_request = request.POST.get('signed_request')
        data = facebook_parse_signed_request(signed_request)
        logger.info('Parsed data:')
        logger.info(data)
        user_id = data['user_id']
        FacebookUser.objects.filter(ext_user_id=user_id).delete()
        logger.info('FacebookDeleteView END ------------------------------')
        return data


class AccountKitRedirectView(OAuthRedirectViewBase):
    serializer_class = AccountKitAuthTokenSerializer


class FacebookRedirectView(OAuthRedirectViewBase):
    serializer_class = FacebookAuthTokenSerializer


class GoogleRedirectView(OAuthRedirectViewBase):
    serializer_class = GoogleAuthTokenSerializer



from django.conf import settings
from django.contrib import admin
from .models import AccountKitUser, AccountKitAccessToken, FacebookAccessToken, FacebookUser, GoogleUser, \
    GoogleAccessToken


class JauthAdminBase(admin.ModelAdmin):
    pass


class AccountKitAdmin(JauthAdminBase):
    exclude = ()


class FacebookAdmin(JauthAdminBase):
    exclude = ()


class GoogleAdmin(JauthAdminBase):
    exclude = ()


admin.site.register(AccountKitUser, AccountKitAdmin)
admin.site.register(AccountKitAccessToken, AccountKitAdmin)
admin.site.register(FacebookUser, FacebookAdmin)
admin.site.register(FacebookAccessToken, FacebookAdmin)
admin.site.register(GoogleUser, GoogleAdmin)
admin.site.register(GoogleAccessToken, GoogleAdmin)

required_params = ['JAUTH_AUTHENTICATION_ERROR_REDIRECT', 'JAUTH_AUTHENTICATION_SUCCESS_REDIRECT']
for p in required_params:
    if not hasattr(settings, p):
        raise Exception('{} configuration missing'.format(p))



import base64
import hashlib
import hmac
import json
import logging
import requests
from django.conf import settings
from requests import Response
from rest_framework.exceptions import ValidationError


logger = logging.getLogger(__name__)


def http_request(method: str, url: str, **kwargs) -> Response:
    res = requests.request(method, url, **kwargs)
    logger.info('{} {} -------------------------------------------------------------------- '.format(method.upper(), url))
    logger.info('{}'.format(res.request.body))
    logger.info('HTTP {} -------------------------------------------------------------------- '.format(res.status_code))
    logger.info('{}'.format(res.text))
    return res


def account_kit_get_access_token(code: str) -> Response:
    """
    GET https://graph.accountkit.com/v1.3/access_token?grant_type=authorization_code&code=<authorization_code>&access_token=AA|<facebook_app_id>|<app_secret>
    :param code:
    :return: dict
    """
    app_id = settings.ACCOUNT_KIT_APP_ID
    app_secret = settings.ACCOUNT_KIT_APP_SECRET
    url = settings.ACCOUNT_KIT_API_URL + '/access_token?grant_type=authorization_code&code={}&access_token=AA|{}|{}'.format(code, app_id, app_secret)
    res = http_request('get', url)
    return res


def account_kit_me(access_token: str) -> Response:
    """
    GET https://graph.accountkit.com/v1.3/me/?access_token=<access_token>
    :param access_token:
    :return: str
    """
    url = settings.ACCOUNT_KIT_API_URL + '/me?access_token={}'.format(access_token)
    res = http_request('get', url)
    return res


def facebook_get_access_token(code: str) -> Response:
    """
    :param code:
    :return: dict
    """
    app_id = settings.FACEBOOK_APP_ID
    app_secret = settings.FACEBOOK_APP_SECRET
    redirect_url = settings.FACEBOOK_REDIRECT_URL
    url = settings.FACEBOOK_API_URL + '/oauth/access_token?client_id={}&redirect_uri={}&client_secret={}&code={}'.format(app_id, redirect_url, app_secret, code)
    res = http_request('get', url)
    return res


def facebook_me(access_token: str) -> Response:
    """
    :param access_token:
    :return: str
    """
    url = settings.FACEBOOK_API_URL + '/me?access_token={}'.format(access_token)
    res = http_request('get', url)
    return res


def facebook_parse_signed_request(signed_request: str) -> dict:
    encoded_sig, payload = signed_request.split('.', 2)
    app_secret = settings.FACEBOOK_APP_SECRET
    data = json.loads(base64.b64decode(payload.replace('-_', '+/')))
    expected_sig = hmac.new(app_secret, msg=payload, digestmod=hashlib.sha256).digest()
    if expected_sig != encoded_sig:
        raise ValidationError("Expected signature does not match encoded signature in Facebook request")
    return data


def google_get_access_token(code: str) -> Response:
    """
    :param code:
    :return: dict
    """
    app_id = settings.GOOGLE_APP_ID
    app_secret = settings.GOOGLE_APP_SECRET
    redirect_url = settings.GOOGLE_REDIRECT_URL
    url = settings.GOOGLE_API_URL + '/oauth2/v4/token'
    data = {
        'code': code,
        'client_id': app_id,
        'client_secret': app_secret,
        'redirect_uri': redirect_url,
        'grant_type': 'authorization_code',
    }
    res = http_request('post', url, data=data)
    return res


def google_me(access_token: str) -> Response:
    """
    {
      "id": "107873301371187782400",
      "email": "kajala@gmail.com",
      "verified_email": true,
      "name": "Jani Kajala",
      "given_name": "Jani",
      "family_name": "Kajala",
      "picture": "https://lh4.googleusercontent.com/-g1HKAdB0pi0/AAAAAAAAAAI/AAAAAAACcbQ/pCyIjkFBHv8/photo.jpg",
      "locale": "en"
    }
    :param access_token:
    :return: str
    """
    url = settings.GOOGLE_API_URL + '/oauth2/v1/userinfo?alt=json&access_token={}'.format(access_token)
    res = http_request('get', url)
    return res
